/*
 * Created on May 10, 2005
 */
package edu.sc.seis.cormorant.seismogramServer;

import java.io.IOException;
import java.sql.SQLException;
import org.apache.log4j.BasicConfigurator;
import edu.iris.Fissures.FissuresException;
import edu.iris.Fissures.IfNetwork.Channel;
import edu.iris.Fissures.IfSeismogramDC.LocalSeismogram;
import edu.iris.Fissures.IfSeismogramDC.RequestFilter;
import edu.iris.Fissures.model.MicroSecondDate;
import edu.iris.Fissures.network.ChannelIdUtil;
import edu.iris.Fissures.seismogramDC.LocalSeismogramImpl;
import edu.sc.seis.cormorant.seismogram.SodDBSeismogramServer;
import edu.sc.seis.fissuresUtil.database.ConnMgr;
import edu.sc.seis.fissuresUtil.database.JDBCTearDown;
import edu.sc.seis.fissuresUtil.database.seismogram.JDBCSeismogramFiles;
import edu.sc.seis.fissuresUtil.display.SimplePlotUtil;
import edu.sc.seis.fissuresUtil.mockFissures.IfNetwork.MockChannel;
import edu.sc.seis.fissuresUtil.xml.SeismogramFileTypes;

/**
 * @author fenner Used to test SodDBSeismogramServer
 */
public class SodDBSeismogramServerTest_Sac extends JDBCTearDown {

    public SodDBSeismogramServerTest_Sac() {
        BasicConfigurator.configure();
    }

    public void setUp() throws SQLException, IOException {
        chan = MockChannel.createChannel();
        jdbcSeisFiles = new JDBCSeismogramFiles(ConnMgr.createConnection());
        sodDBSeisServer = new SodDBSeismogramServer(jdbcSeisFiles, false);
        seis1 = SimplePlotUtil.createSpike(new MicroSecondDate(START_TIME));
        seis2 = SimplePlotUtil.createSpike(new MicroSecondDate(START_TIME
                + ONE_SECOND));
        seis3 = SimplePlotUtil.createSpike(new MicroSecondDate(START_TIME
                - ONE_SECOND));
        filePath = "POND_II/Event/data/II.PFO.00.TEST.sac";
    }

    public void testAvailable_DataWithNoSeismogramAndNoQuery()
            throws SQLException {
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(new RequestFilter[0]);
        assertEquals(0, testResultsArray.length);
    }

    public void testAvailable_DataWithOneSeismogramAndNoQuery()
            throws Exception {
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(new RequestFilter[0]);
        assertEquals(0, testResultsArray.length);
    }

    public void testAvailable_DataWithOneSeismogramAndOneQuery()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(1, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
    }

    public void testAvailable_DataWithTwoSeismogramsAndOneQuery()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH + 1000).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(2, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[1].channel_id,
                                          chan.get_id()));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
    }

    public void testAvailable_DataWithTwoSeismogramsAndTwoQueries()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[2];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH + 1000).getFissuresTime());
        requestArray[1] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH + 1000).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(4, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[1].channel_id,
                                          chan.get_id()));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[2].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[2].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[2].end_time));
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[3].channel_id,
                                          chan.get_id()));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[3].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[3].end_time));
    }

    public void testAvailable_DataWithThreeSeismogramsAndOneQuerie_SplitBeginningCase()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis3,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME
                                                    + (ONE_SECOND / 2)).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + +SEIS_LENGTH
                                                    + (ONE_SECOND * 10)).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(3, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
        assertEquals(seis3.getBeginTime(),
                     new MicroSecondDate(testResultsArray[2].start_time));
        assertEquals(seis3.getEndTime(),
                     new MicroSecondDate(testResultsArray[2].end_time));
    }

    public void testAvailable_DataWithThreeSeismogramsAndOneQuerie_SplitEndingCase()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis3,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME
                                                    - (ONE_SECOND * 10)).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH
                                                    - (ONE_SECOND / 2)).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(3, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
        assertEquals(seis3.getBeginTime(),
                     new MicroSecondDate(testResultsArray[2].start_time));
        assertEquals(seis3.getEndTime(),
                     new MicroSecondDate(testResultsArray[2].end_time));
    }

    public void testAvailable_DataWithThreeSeismogramsAndOneQuerie_NoSplitCase()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis3,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME
                                                    - (ONE_SECOND * 10)).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH
                                                    + (ONE_SECOND * 10)).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(3, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
        assertEquals(seis3.getBeginTime(),
                     new MicroSecondDate(testResultsArray[2].start_time));
        assertEquals(seis3.getEndTime(),
                     new MicroSecondDate(testResultsArray[2].end_time));
    }

    public void testAvailable_DataWithThreeSeismogramsAndOneQuerie_SplitBothCase()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis2,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis3,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        RequestFilter[] requestArray = new RequestFilter[1];
        requestArray[0] = new RequestFilter(chan.get_id(),
                                            new MicroSecondDate(START_TIME
                                                    + (ONE_SECOND / 2)).getFissuresTime(),
                                            new MicroSecondDate(START_TIME
                                                    + SEIS_LENGTH
                                                    - (ONE_SECOND / 2)).getFissuresTime());
        RequestFilter[] testResultsArray = sodDBSeisServer.available_data(requestArray);
        assertEquals(3, testResultsArray.length);
        assertTrue(ChannelIdUtil.areEqual(testResultsArray[0].channel_id,
                                          chan.get_id()));
        assertEquals(seis1.getBeginTime(),
                     new MicroSecondDate(testResultsArray[0].start_time));
        assertEquals(seis1.getEndTime(),
                     new MicroSecondDate(testResultsArray[0].end_time));
        assertEquals(seis2.getBeginTime(),
                     new MicroSecondDate(testResultsArray[1].start_time));
        assertEquals(seis2.getEndTime(),
                     new MicroSecondDate(testResultsArray[1].end_time));
        assertEquals(seis3.getBeginTime(),
                     new MicroSecondDate(testResultsArray[2].start_time));
        assertEquals(seis3.getEndTime(),
                     new MicroSecondDate(testResultsArray[2].end_time));
    }

    public void testRetrieve_SeismogramsWithNoSeismogramAndNoQuery()
            throws SQLException, FissuresException {
        LocalSeismogram[] testResultsArray = sodDBSeisServer.retrieve_seismograms(new RequestFilter[0]);
        assertEquals(0, testResultsArray.length);
    }

    public void testRetrieve_SeismogramsWithOneSeismogramAndNoQuery()
            throws Exception {
        jdbcSeisFiles.saveSeismogramToDatabase(chan,
                                               seis1,
                                               filePath,
                                               SeismogramFileTypes.SAC);
        LocalSeismogram[] testResultsArray = sodDBSeisServer.retrieve_seismograms(new RequestFilter[0]);
        assertEquals(0, testResultsArray.length);
    }

    private String filePath;

    private LocalSeismogramImpl seis1;

    private LocalSeismogramImpl seis2;

    private LocalSeismogramImpl seis3;

    private SodDBSeismogramServer sodDBSeisServer;

    private JDBCSeismogramFiles jdbcSeisFiles;

    private Channel chan;

    private static final long ONE_SECOND = (1000 * 1000);

    private static final long SEIS_LENGTH = (49 * 1000 * 1000);

    private static final long START_TIME = 1000;
}
